<?php
/**
 * Upseller for Elementor
 * Customizable upsells and related products for Elementor editor
 * Exclusively on https://1.envato.market/upseller-elementor
 *
 * @encoding        UTF-8
 * @version         1.0.2
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmitry Merkulov (dmitry@merkulov.design), Cherviakov Vlad (vladchervjakov@gmail.com)
 * @support         help@merkulov.design
 **/

namespace Merkulove\UpsellerElementor;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Caster class contain main plugin logic.
 *
 * @since 1.0.0
 *
 **/
final class Caster {

	/**
	 * The one true Caster.
	 *
     * @since 1.0.0
     * @access private
	 * @var Caster
	 **/
	private static $instance;

    /**
     * Setup the plugin.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function setup() {

        /** Define hooks that runs on both the front-end as well as the dashboard. */
        $this->both_hooks();

        /** Define public hooks. */
        $this->public_hooks();

        /** Define admin hooks. */
        $this->admin_hooks();

    }

    /**
     * Define hooks that runs on both the front-end as well as the dashboard.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function both_hooks() {

        /** Hooks which should work everywhere in backend and frontend. */
        /** Register WooCommerce frontend */
        if ( $this->is_active_woocommerce() ) {
            add_action( 'init', [$this, 'register_wc_frontend'], 5 );
        }

    }

    /**
     * Register all of the hooks related to the public-facing functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function public_hooks() {

        /** Work only on frontend area. */
        if ( is_admin() ) { return; }

        /** Public hooks and filters. */

    }

    /**
     * Register all of the hooks related to the admin area functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function admin_hooks() {

        /** Work only in admin area. */
        if ( ! is_admin() ) { return; }

        /** Admin hooks and filters here. */

    }

	/**
	 * This method used in register_activation_hook
	 * Everything written here will be done during plugin activation.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function activation_hook() {

		/** Activation hook */

	}

    /**
     * Checks if woocommerce is active
     *
     * @since 1.0.0
     * @access public
     *
     * @return bool
     **/
    public function is_active_woocommerce() {

        $woocommerce_active = false;
        if (  in_array('woocommerce/woocommerce.php',
            apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
            $woocommerce_active = true;
        }

        return $woocommerce_active;
    }

    /**
     * Register WooCommerce frontend
     *
     * @since 1.0.0
     * @access public
     */
    public function register_wc_frontend() {
        wc()->frontend_includes();
    }


    /**
     * Hide product parts actions.
     *
     * @param $settings
     * @param $args
     * @return void
     * @since 1.0.0
     * @access private
     */
    public function remove_product_actions( $settings, $args ) {
        if ( $settings[$args['show_cart_key']] !== 'yes' ) {
            remove_action( 'woocommerce_after_shop_loop_item', 'woocommerce_template_loop_add_to_cart');
            remove_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_add_to_cart');
        }

        if ( $settings[$args['show_price_key']] !== 'yes' ) {
            remove_action(
                'woocommerce_after_shop_loop_item_title',
                'woocommerce_template_loop_price',
                10
            );
        }
    }


    /**
     * Restore defaults actions.
     *
     * @param $settings
     * @param $args
     * @return void
     * @since 1.0.0
     * @access private
     */
    public function restore_product_actions( $settings, $args ) {
        if ( $settings[$args['show_cart_key']] !== 'yes' ) {
            add_action( 'woocommerce_after_shop_loop_item', 'woocommerce_template_loop_add_to_cart');
            add_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_add_to_cart');
        }

        if ( $settings[$args['show_price_key']] !== 'yes' ) {
            add_action(
                'woocommerce_after_shop_loop_item_title',
                'woocommerce_template_loop_price',
                10
            );
        }
    }


    /**
	 * Main Caster Instance.
	 * Insures that only one instance of Caster exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Caster
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
